****************************************************************************
*                                                                          *
* AUTHOR:       R. Scott Mackay / Tom Kallal                               *
*                                                                          *
* PROGRAM-ID:   Pop_DBF - Version 2.02 February 16, 1994                   *
*                                                                          *
* DESCRIPTION:  POP Tables, Indexes, and Relations - Scan any open areas,  *
*               restoring DBF(), ALIAS(), Exclusive/Shared, ReadOnly (from *
*               api routine), IdxList, A/Descending, ORDER(), FILTER(),    *
*               Relations, Skip, and RECNO() for each table.  Information  *
*               is obtained from a public array _R_Stat that was stored by *
*               Push_DBF and released with the last call to Pop_DBF.       *
*                                                                          *
*               NOTE:  The following API libraries must be loaded prior    *
*                      to using Push_DBF & Pop_DBF:                        *
*                                                                          *
*                      DOS:     FPath.PLB (Microsoft - usually located in  *
*                               the FoxApp directory) AND R_Status.PLB.    *
*                                                                          *
*                      WINDOWS: FoxTools.FLL OR FPath.FLL (both Microsoft) *
*                               AND R_Status.FLL.                          *
*                                                                          *
****************************************************************************
PARAMETERS HOLD, CURRENT, RELATIONS		&& HOLD      - Hold "stack"
																			&& CURRENT   - Current table only
																			&& RELATIONS - Relations & Index only

PRIVATE LastArea, _R_Idx, Tries, AllOpen, ;
	Cur_Area, OpenError, ChildIndex, ChildTable, ;
	UseError, OldOnErr, Rel_Index, Cur_Rel, Cur_Tgt, ;
	Tag_Index, Cur_AD, HoldArea

#DEFINE Old_DBF    _R_Stat[m._R_Idx+m.Cur_Area,01]
#DEFINE Old_Alias  _R_Stat[m._R_Idx+m.Cur_Area,02]
#DEFINE Old_EX     _R_Stat[m._R_Idx+m.Cur_Area,03]
#DEFINE Old_RO     _R_Stat[m._R_Idx+m.Cur_Area,04]
#DEFINE Old_Index  _R_Stat[m._R_Idx+m.Cur_Area,05]
#DEFINE Old_AD     _R_Stat[m._R_Idx+m.Cur_Area,06]
#DEFINE Old_Order  _R_Stat[m._R_Idx+m.Cur_Area,07]
#DEFINE Old_Filter _R_Stat[m._R_Idx+m.Cur_Area,08]
#DEFINE Old_Rel    _R_Stat[m._R_Idx+m.Cur_Area,09]
#DEFINE Old_Skip   _R_Stat[m._R_Idx+m.Cur_Area,10]
#DEFINE Old_Recno  _R_Stat[m._R_Idx+m.Cur_Area,11]
#DEFINE Req_Tables _R_Stat[m._R_Idx+m.Cur_Area,12]
#DEFINE Old_Area   _R_Stat2[m._R_Count+1]


********** SETUP SECTION **********
***** TRAP POP WITHOUT MATCHING PUSH *****
IF TYPE("m._R_Count")='U' OR m._R_Count<0
	RETURN
ENDIF

***** TRAP INVALID PARAMETERS *****
IF TYPE("m.HOLD")#'L' OR TYPE("m.CURRENT")#'L' ;
		OR TYPE("m.RELATIONS")#'L'
	WAIT WINDOW "Invalid Parameter(s)"
	RETURN
ENDIF

***** TRAP ABSCENSE OF LIBRARIES *****
IF TYPE("WORDNUM(' ',1)")='U' OR TYPE("R_Status()")='U'
	WAIT WINDOW "Can't Find one or more of the"+CHR(13)+ ;
							"following Libraries:         "+CHR(13)+ ;
							"                             "+CHR(13)+ ;
							"Foxtools or FPath, R_Status. "
	RETURN
ENDIF

***** PREVENT INTERRUPTION *****
PUSH KEY CLEAR
=Push_Set("ESCAPE", "OFF")

***** MISC SETUP *****
LastArea  = IIF(!_DOS OR 'X' $ VERSION(), 225, 25)
_R_Idx    = m._R_Count*m.LastArea
HoldArea  = SELECT(0)


********** POP TABLE ENVIRONMENT **********
***** OPEN TABLES IN CORRECT AREAS - ALIASED CHILDREN FIRST *****
IF !m.Relations
	Tries   = 0
	AllOpen = .F.
	DO WHILE m.Tries<100 AND !m.AllOpen
		AllOpen=.T.
		FOR Cur_Area=IIF(m.CURRENT, SELECT(0), 1) ;
				TO IIF(m.CURRENT, SELECT(0), m.LastArea)

			*** PARENT'S INDEX EXPRESSION CONTAINS ONE OR MORE ALIASED FIELDS, ***
			*** SO MAKE SURE ALIASED CHILD TABLES ARE ALREADY OPEN.            ***
			OpenError=.F.
			IF !EMPTY(Req_Tables)
				ChildIndex = 1
				ChildTable = WORDNUM(Req_Tables, ;
					m.ChildIndex, "||")
				DO WHILE !EMPTY(m.ChildTable)
					IF !USED(m.ChildTable)
						OpenError=.T.															&& Child not opened
						EXIT
					ENDIF
					ChildIndex = m.ChildIndex+1
					ChildTable = WORDNUM(Req_Tables, m.ChildIndex, "||")
				ENDDO
			ENDIF
			IF m.OpenError AND m.Tries<100
				AllOpen=.F.
				LOOP
			ENDIF

			*** CHECK TO SEE IF DBF IS OPEN THE SAME WAY IT WAS. ***
			*** (ie. DBF() ALIAS(), EXCLUSIVE/SHARED, READONLY)  ***
	 		IF !EMPTY(Old_DBF)
				SELECT (m.Cur_Area)
				IF !DBF()==Old_DBF OR !ALIAS()==Old_Alias OR;
						!Old_EX==IIF(SYS(2011)="Exclusive", ;
						"EXCLUSIVE", "SHARED") OR !Old_RO==R_Status()
					UseError=.F.
					OldOnErr=ON("ERROR")
					ON ERROR UseError=.T.
					** Use a case to avoid slow macro expansion
					DO CASE
					CASE Old_EX=="SHARED"    AND Old_RO==""						&& Shared/Write
						USE (Old_DBF) AGAIN ALIAS (Old_Alias) ;
							SHARED
					CASE Old_EX=="SHARED"    AND Old_RO=="NOUPDATE"		&& Shared/ROnly
						USE (Old_DBF) AGAIN ALIAS (Old_Alias) ;
							SHARED NOUPDATE
					CASE Old_EX=="EXCLUSIVE" AND Old_RO==""						&& Exclsv/Write
						USE (Old_DBF) AGAIN ALIAS (Old_Alias) ;
							EXCLUSIVE
					CASE Old_EX=="EXCLUSIVE" AND Old_RO=="NOUPDATE"		&& Exclsv/ROnly
						USE (Old_DBF) AGAIN ALIAS (Old_Alias) ;
							EXCLUSIVE NOUPDATE
					ENDCASE					
					IF EMPTY(m.OldOnErr)
						ON ERROR																	&& Avoid macro expand
					ELSE
						ON ERROR &OldOnErr
					ENDIF
					IF m.UseError
						USE
					ENDIF
				ENDIF
			ELSE
				IF !EMPTY(ALIAS(m.Cur_Area))
					USE IN m.Cur_Area
				ENDIF
			ENDIF

		ENDFOR
		Tries=m.Tries+1
	ENDDO
ENDIF

***** SET RELATIONS, SKIP, IDXLIST, ORDER, A/DESCENDING, AND FILTER *****
FOR Cur_Area=IIF(m.CURRENT, SELECT(0), 1) TO ;
		IIF(m.CURRENT, SELECT(0), m.LastArea)
	IF !EMPTY(ALIAS(m.Cur_Area))												&& Old area Old Alias
		SELECT (m.Cur_Area)

		*** RESET RELATIONS ***
		SET RELATION TO																		&& Clear relations
		IF !EMPTY(Old_Rel)
			Rel_Index = 1
			Cur_Rel   = WORDNUM(Old_Rel, m.Rel_Index,  "||")
			Cur_Tgt   = WORDNUM(Old_Rel, m.Rel_Index+1,"||")
			DO WHILE !EMPTY(m.Cur_Rel)
				IF USED(m.Cur_Tgt)
					IF EMPTY(ORDER(m.Cur_Tgt))
						SET ORDER TO 1 IN (m.Cur_Tgt)
					ENDIF
					SET RELATION TO &Cur_Rel INTO ;
						(m.Cur_Tgt) ADDITIVE
				ENDIF
				Rel_Index = Rel_Index + 2
				Cur_Rel = WORDNUM(Old_Rel, m.Rel_Index,  "||")
				Cur_Tgt = WORDNUM(Old_Rel, m.Rel_Index+1,"||")
			ENDDO
		ENDIF

		*** RESET RECNO - LEAVING CHILD RECNO's WHERE THEY WERE ***
		DO Rec_Set																				&& Record Number
		
		*** RESET SKIP ***
		IF EMPTY(Old_Skip)
			SET SKIP TO																			&& Avoid macro expand
		ELSE
			SET SKIP TO &Old_Skip														&& Set SKIP TO
		ENDIF

		*** RESET INDEX LIST ***
		IF EMPTY(Old_Index)
			SET INDEX TO																		&& Avoid macro expand
		ELSE
			** IF INDEX LIST HAS CHANGED, RESTORE OLD **
			IF !Old_Index==IdxList()
				SET INDEX TO &Old_Index
			ENDIF
		ENDIF

		*** RESET ASCEND/DESCEND STATUS FOR ALL TAGS ***
		Tag_Index = 1
		Cur_AD    = WORDNUM(Old_AD, m.Tag_Index, "||")
		DO WHILE !EMPTY (m.Cur_AD)
			IF m.Cur_AD=="DESCENDING"
				SET ORDER TO (m.Tag_Index) DESCENDING
			ELSE
				SET ORDER TO (m.Tag_Index) ASCENDING
			ENDIF
			Tag_Index = Tag_Index+1
			Cur_AD    = WORDNUM(Old_AD, m.Tag_Index, "||")
		ENDDO

		*** RESET ORDER ***
		SET ORDER TO (Old_Order)

		*** RESET FILTER ***
		IF EMPTY(Old_Filter)
			SET FILTER TO																		&& Avoid macro expand
		ELSE
			SET FILTER TO &Old_Filter
		ENDIF

	ENDIF
ENDFOR


********** CLEANUP SECTION **********
** Select the OLD AREA they were in when Push_DBF
** was invoked.  Unless only relations are being
** restored.
**
IF m.RELATIONS
	SELECT (m.HoldArea)
ELSE
	IF !m.CURRENT
		SELECT (Old_Area)
	ENDIF
ENDIF

***** 'POP' INFORMATION OFF ARRAY & FREE MEMORY, UNLESS 'HOLD' *****
IF !m.Hold
	_R_Count=m._R_Count-1
	IF m._R_Count>=0
		DIMENSION _R_Stat [(m._R_Count+1)*m.LastArea, 12]
		DIMENSION _R_Stat2[m._R_Count+1]
	ELSE
		*** IF LAST POP, RELEASE PUBLIC VARIABLES ***
		RELEASE _R_Count, _R_Stat, _R_Stat2
	ENDIF
ENDIF

=Pop_Set("ESCAPE")
POP KEY

RETURN




****************************************************************************
************************** FUNCTIONS & PROCEDURES **************************
****************************************************************************

********** REC_SET - SET RECORD POINTER (CALLED RECURSIVELY) **********
*                                                                     *
* NOTE: Once a relation is set, the record pointers, for the current  *
*       relation target and all other children of the current parent  *
*       already set in a relations, are reset based on the relations. *
*       This behavior makes it difficult to restore the environment   *
*       so that record pointers are exactly where they were when      *
*       Push_DBF was called.                                          *
*       The Solution to this problem is to reset all record pointers  *
*       for the current parent and all children after the             *
*       relationships have been set.                                  *
*       This procedure gets called recursively so that all the record *
*       pointers under the current relation 'tree' get reset.         *
*                                                                     *
***********************************************************************
PROCEDURE Rec_Set
PRIVATE Cur_Area, Tgt_Index, Cur_Tgt

Cur_Area=SELECT(0)

IF !EMPTY(Old_Recno) AND RECCOUNT()>0
	IF Old_Recno>RECCOUNT()
		GO BOTTOM
		IF !EOF()
			SKIP
		ENDIF
	ELSE
		IF Old_Recno<1
			GO TOP
		ELSE
			GO Old_Recno																		&& Reposition Record
		ENDIF
	ENDIF
ENDIF

IF !EMPTY(Old_Rel) AND !EMPTY(RELATION(1))
	Tgt_Index = 1
	Cur_Tgt   = WORDNUM(Old_Rel, m.Tgt_Index+1, "||")
	DO WHILE !EMPTY (m.Cur_Tgt)
		SELECT (m.Cur_Tgt)
		DO Rec_Set																				&& ==> RECURSIVE CALL
		SELECT (m.Cur_Area)
		Tgt_Index = m.Tgt_Index+2
		Cur_Tgt   = WORDNUM(Old_Rel, m.Tgt_Index+1, "||")
	ENDDO
ENDIF
RETURN



********** IDXLIST - BUILD INDEX LIST **********
FUNCTION IdxList
PRIVATE ALL

IdxStr  = ""
Cur_NDX = 1
DO WHILE !EMPTY(NDX(m.Cur_NDX))
	IdxStr  = m.IdxStr+NDX(m.Cur_NDX)+", "
	Cur_NDX = m.Cur_NDX+1
ENDDO
Cur_CDX = 1
DO WHILE !EMPTY(CDX(m.Cur_CDX))
	IF !STRTRAN(CDX(m.Cur_CDX), "CDX", "DBF")==DBF()
		IdxStr  = m.IdxStr+CDX(m.Cur_CDX)+", "
	ENDIF
	Cur_CDX = m.Cur_CDX+1
ENDDO
IF !EMPTY(m.IdxStr)
	IdxStr=LEFT(m.IdxStr, LEN(m.IdxStr)-2)
ENDIF
RETURN m.IdxStr
