****************************************************************************
*                                                                          *
* AUTHOR:       R. Scott Mackay                                            *
*                                                                          *
* PROGRAM-ID:   Pop_Ord.PRG - February 1994                                *
*               Revised 05/23/94 - Allow numeric parameter for work area.  *
*                                                                          *
* DESCRIPTION:  When used after Push_Ord, allows you to reset the previous *
*               index order 'pushed' in the specified work area.           *
*               Results are retrieved from a public variable 'gcOrdList'.  *
*                                                                          *
* SYNTAX:       Pop_Ord([<expC>])                                          *
* RETURNS:      Logical (was pop successfull?)                             *
* PARAMETERS:   expC - Alias to pop order in                               *
*                                                                          *
* EXAMPLE:      =Pop_Ord()         && Restore last 'pushed' order          *
*       or          =Pop_Ord("States") && Restore last 'pushed' states order   *
*   or          =Pop_Ord(5)        && Restore last 'pushed' order in 5     *
*                                                                          *
****************************************************************************
PARAMETERS lcAlias
PRIVATE lnObjLoc, lnObjLen, lcObject, lcOldOrd

IF TYPE("m.gcOrdList")='U'
	RETURN .F.                                                                                                                                                                      && Push w/o prior Pop
ENDIF

lcAlias = UPPER(ALLTRIM(IIF(PARAMETERS()=1,    ;
	IIF(TYPE("m.lcAlias")='N', ALIAS(m.lcAlias), ;
	UPPER(m.lcAlias)), ALIAS())))

***** GET LAST ORDER *****
lnObjLoc = RAT('|'+m.lcAlias, UPPER(m.gcOrdList))
IF m.lnObjLoc=0
	RETURN .F.                                                                                                                                                                      && Push w/o prior Pop
ELSE
	lnObjLen = AT(',', SUBSTR(m.gcOrdList, m.lnObjLoc))
	lcObject = SUBSTR(m.gcOrdList, ;
		m.lnObjLoc, m.lnObjLen-1)
	lcOldOrd = WORDNUM(m.lcObject, 2, '~')
ENDIF

***** REMOVE ORDER SETTING FROM THE LIST *****
IF m.lnObjLoc+m.lnObjLen-1 = LEN(m.gcOrdList)
	gcOrdList = SUBSTR(m.gcOrdList, 1, m.lnObjLoc-1)
ELSE
	gcOrdList = SUBSTR(m.gcOrdList, 1, m.lnObjLoc-1)+ ;
		SUBSTR(m.gcOrdList, m.lnObjLoc+m.lnObjLen)
ENDIF

***** SET LAST ORDER *****
IF USED(m.lcAlias)
	SET ORDER TO (m.lcOldOrd) IN (m.lcAlias)
ELSE
	RETURN .F.                                                                                                                                                                      && Pop Failure
ENDIF

RETURN
