****************************************************************************
*                                                                          *
* AUTHOR:       Richard Grossman (213) 547-2191                            *
*               Database Advisor April 92, Page 42                         *
*               Modified 02/94 by R. Scott Mackay                          *
*                                                                          *
* PROGRAM-ID:   Pop_Set.PRG                                                *
*                                                                          *
* DESCRIPTION:  When used after Push_Set, allows you to restore the        *
*               setting of any ON/OFF SET command to its previous setting. *
*               Results are retrieved from a public variable 'gcSetList'.  *
*                                                                          *
* SYNTAX:       Pop_Set(<expC>)                                            *
* RETURNS:      Logical (was pop valid/successful?)                        *
* PARAMETERS:   expC - SET Object (i.e. 'BELL', 'CLOCK')                   *
*                                                                          *
* EXAMPLE:      jlRetVal=Pop_Set("CLOCK")                                  *
*                                                                          *
****************************************************************************
PARAMETERS lcObject

PRIVATE lcThisSet, lnObjLoc, lnObjLen, lcObjSet

IF TYPE("m.gcSetList")='U' OR TYPE('m.lcObject')#'C' ;
		OR EMPTY(m.lcObject) OR EMPTY(SET(m.lcObject))
	RETURN .F.																					&& Invalid parameters
ENDIF

lcObject = UPPER(ALLTRIM(lcObject))

** Find last occurrence of the object in the list,
** and its setting:
lnObjLoc = RAT(m.lcObject, m.gcSetList)
IF m.lnObjLoc=0
	RETURN .F.																					&& Push w/o prior Pop
ELSE
	lnObjLen = AT('=', SUBSTR(m.gcSetList, m.lnObjLoc))
	lcObjSet = ALLTRIM(SUBSTR(m.gcSetList, ;
		m.lnObjLoc+m.lnObjLen, 3))

	*** SET THE OBJECT TO ITS PREVIOUS STATE ***
	** Attempt to avoid macro expansion by testing for
	** some of the more common SET expressions.
	**
	DO CASE
	CASE m.lcObject=="ESCAPE"
		IF m.lcObjSet=="ON"
			SET ESCAPE ON
		ELSE
			SET ESCAPE OFF
		ENDIF
	CASE m.lcObject=="EXACT"
		IF m.lcObjSet=="ON"
			SET EXACT ON
		ELSE
			SET EXACT OFF
		ENDIF
	CASE m.lcObject=="NEAR"
		IF m.lcObjSet=="ON"
			SET NEAR ON
		ELSE
			SET NEAR OFF
		ENDIF
	CASE m.lcObject=="TALK"
		IF m.lcObjSet=="ON"
			SET TALK ON
		ELSE
			SET TALK OFF
		ENDIF
	OTHERWISE
		IF m.lcObjSet=="ON"
			SET &lcObject ON
		ELSE
			SET &lcObject OFF
		ENDIF
	ENDCASE

	*** REMOVE THE OBJECT AND ITS SETTING FROM THE LIST ***
	IF m.lnObjLoc+m.lnObjLen+3 = LEN(m.gcSetList)
		gcSetList = SUBSTR(m.gcSetList, 1, m.lnObjLoc-1)
	ELSE
		gcSetList = SUBSTR(m.gcSetList, 1, m.lnObjLoc-1)+ ;
			SUBSTR(m.gcSetList, m.lnObjLoc+m.lnObjLen+4)
	ENDIF
ENDIF
RETURN
