****************************************************************************
*                                                                          *
* AUTHOR:       R. Scott Mackay / Tom Kallal                               *
*                                                                          *
* PROGRAM-ID:   Push_DBF - Version 2.02 February 16, 1994                  *
*                                                                          *
* DESCRIPTION:  PUSH Tables, Indexes, and Relations - Scan any open areas, *
*               saving DBF(), ALIAS(), Exclusive/Shared, ReadOnly (from    *
*               api routine), IdxList, ORDER(), RECNO(), FILTER(), and     *
*               Relations for each table.  Results are stored in a public  *
*               array _R_Stat that is released with the last call to       *
*               Pop_DBF.                                                   *
*                                                                          *
*               NOTE:  The following API libraries must be loaded prior    *
*                      to using Push_DBF & Pop_DBF:                        *
*                                                                          *
*                      DOS:     FPath.PLB (Microsoft - usually located in  *
*                               the FoxApp directory) AND R_Status.PLB.    *
*                                                                          *
*                      WINDOWS: FoxTools.FLL OR FPath.FLL (both Microsoft) *
*                               AND R_Status.FLL.                          *
*                                                                          *
****************************************************************************
PARAMETERS RESET, CLOSE, XTREE	&& RESET - Clear Order, Filter, & Relations
                       					&& CLOSE - Close Tables
                       					&& XTREE - Close Tables except eXclude Tree

PRIVATE LastArea, _R_Idx, Cur_Area, ;
	EX_Share, ExclTree


******* SETUP SECTION *******
***** TRAP INVALID PARAMETERS *****
IF TYPE("m.RESET")#'L' OR TYPE("m.CLOSE")#'L'
	WAIT WINDOW "Invalid Parameter(s)"
	RETURN
ENDIF

***** TRAP ABSCENSE OF LIBRARIES *****
IF TYPE("WORDNUM(' ', 1)")='U' OR TYPE("R_Status()")='U'
	WAIT WINDOW "Can't Find one or more of the"+CHR(13)+ ;
							"following Libraries:         "+CHR(13)+ ;
							"                             "+CHR(13)+ ;
							"Foxtools or FPath, R_Status. "
	RETURN
ENDIF

***** PREVENT INTERRUPTION *****
PUSH KEY CLEAR
=Push_Set("ESCAPE", "OFF")

LastArea = IIF(!_DOS OR 'X' $ VERSION(), 225, 25)

IF TYPE("m._R_Stat")='U'
	PUBLIC _R_Count
	PUBLIC ARRAY _R_Stat[1], _R_Stat2[1]
	_R_Count=-1
ENDIF

_R_Count=m._R_Count+1
DIMENSION _R_Stat [(m._R_Count+1)*m.LastArea, 12]
DIMENSION _R_Stat2[m._R_Count+1]


******* PUSH TABLE ENVIRONMENT *******
_R_Stat2[m._R_Count+1]=SELECT(0)											&& Save current area
_R_Idx=m._R_Count*m.LastArea	
FOR Cur_Area = 1 TO m.LastArea

	IF EMPTY(ALIAS(m.Cur_Area))
		LOOP
	ENDIF

	SELECT (m.Cur_Area)
	EX_Share=IIF(SYS(2011)="Exclusive", "EXCLUSIVE", ;
		"SHARED")
	_R_Stat[m._R_Idx+m.Cur_Area, 01] = DBF()						&& Save Table
	_R_Stat[m._R_Idx+m.Cur_Area, 02] = ALIAS()					&& Save Alias
	_R_Stat[m._R_Idx+m.Cur_Area, 03] = m.Ex_Share				&& Save Excluse/Shared
	_R_Stat[m._R_Idx+m.Cur_Area, 04] = R_Status()				&& Save ReadOnly Stat
	_R_Stat[m._R_Idx+m.Cur_Area, 05] = IdxList()				&& Save Index List
	_R_Stat[m._R_Idx+m.Cur_Area, 06] = A_DList() 	  		&& Save A/Descending
	_R_Stat[m._R_Idx+m.Cur_Area, 07] = ORDER()					&& Save Order
	_R_Stat[m._R_Idx+m.Cur_Area, 08] = FILTER()					&& Save Filter Exprssn
	_R_Stat[m._R_Idx+m.Cur_Area, 09] = RelList()				&& Save Relation List
	_R_Stat[m._R_Idx+m.Cur_Area, 10] = SET("SKIP")			&& Save Skip List	
	_R_Stat[m._R_Idx+m.Cur_Area, 11] = RECNO()					&& Save Record Number	
	_R_Stat[m._R_Idx+m.Cur_Area, 12] = ReqList()				&& Save Required Tbls

ENDFOR


******* CLEAR & CLOSE BASED ON PARAMETERS *******
IF m.RESET OR m.CLOSE
	ExclTree=""
	IF m.XTREE
		SELECT (_R_Stat2[m._R_Count+1])										&& Select Prev Area
		IF !EMPTY(ALIAS())
			ExclTree=BuildTree()+"||"												&& Build Exclude Tree
		ENDIF
	ENDIF
	FOR Cur_Area = 1 TO m.LastArea
		IF EMPTY(ALIAS(m.Cur_Area))
			LOOP
		ENDIF
		SELECT (m.Cur_Area)
		** Clear Relations, Filter(), and Order
		IF m.RESET
			IF m.CLOSE AND !".TMP" $ DBF()
				IF !("||"+ALIAS()+"||" $ m.ExclTree)
					USE
				ENDIF
			ELSE
				SET RELATION TO
				SET SKIP TO
				SET FILTER TO
				SET ORDER TO
			ENDIF
		ELSE
			IF m.CLOSE AND !".TMP" $ DBF()									&& Close Table & Index
				IF !("||"+ALIAS()+"||" $ m.ExclTree)
					USE																					&& Unless Current Tree
				ENDIF
			ENDIF
		ENDIF
	ENDFOR
ENDIF


******* CLEANUP SECTION *******
SELECT (_R_Stat2[m._R_Count+1])												&& Select Prev Area

=Pop_Set("ESCAPE")
POP KEY

RETURN




****************************************************************************
************************** FUNCTIONS & PROCEDURES **************************
****************************************************************************

********** IDXLIST - BUILD INDEX LIST **********
FUNCTION IdxList
PRIVATE IdxStr, Cur_NDX, Cur_CDX

IdxStr  = ""
Cur_NDX = 1
DO WHILE !EMPTY(NDX(m.Cur_NDX))
	IdxStr  = m.IdxStr+NDX(m.Cur_NDX)+", "
	Cur_NDX = m.Cur_NDX+1
ENDDO
Cur_CDX = 1
DO WHILE !EMPTY(CDX(m.Cur_CDX))
	IF !STRTRAN(CDX(m.Cur_CDX), "CDX", "DBF")==DBF()
		IdxStr = m.IdxStr+CDX(m.Cur_CDX)+", "
	ENDIF
	Cur_CDX = m.Cur_CDX+1
ENDDO
IF LEN(m.IdxStr)>1
	IdxStr=LEFT(m.IdxStr, LEN(m.IdxStr)-2)
ENDIF
RETURN m.IdxStr



********** A_DLIST - BUILD ASCENDING/DESCENDING LIST **********
FUNCTION A_DList
PRIVATE A_DStr, Old_Ord, TagIdx, TagError, OldOnErr

A_DStr   = ""
Old_Ord  = ORDER()
TagIdx   = 1
TagError = .F.
OldOnErr=ON("ERROR")
ON ERROR TagError=.T.
SET ORDER TO m.TagIdx
DO WHILE !m.TagError
	A_DStr = m.A_DStr+IIF("DESCENDING" $ SET("ORDER"), ;
		"DESCENDING", "ASCENDING")+"||"
	TagIdx = m.TagIdx+1
	SET ORDER TO m.TagIdx
ENDDO
IF LEN(m.A_DStr)>1
	A_DStr=LEFT(m.A_DStr, LEN(m.A_DStr)-2)
ENDIF
SET ORDER TO m.Old_Ord
IF EMPTY(m.OldOnErr)
	ON ERROR
ELSE
	ON ERROR &OldOnErr
ENDIF
RETURN m.A_DStr



********** RELLIST - BUILD LIST OF RELATIONS **********
FUNCTION RelList
PRIVATE RelStr, Cur_Rel

RelStr  = ""
Cur_Rel = 1
DO WHILE !EMPTY(RELATION(m.Cur_Rel))
	RelStr  = m.RelStr+RELATION(m.Cur_Rel)+"||"+ ;
		TARGET(m.Cur_Rel)+"||"
	Cur_Rel = m.Cur_Rel+1
ENDDO
IF LEN(m.RelStr)>1
	RelStr=LEFT(m.RelStr, LEN(m.RelStr)-2)
ENDIF
RETURN m.RelStr



********** REQLIST - BUILD LIST OF REQUIRED ALIASED FIELD TABLES **********
FUNCTION ReqList
PRIVATE ReqStr, Sys14Count, Sys14Exp, TempStr

ReqStr     = ""
Sys14Count = 1
Sys14Exp   = SYS(14, m.Sys14Count)
DO WHILE !EMPTY(m.Sys14Exp)														&& Get Index Expressn
	IF '.' $ Sys14Exp
		TempStr=GetAlias(m.Sys14Exp)											&& Call to GetAlias
		IF !EMPTY(m.TempStr)
			ReqStr=m.ReqStr+m.TempStr												&& Store Child Tables
		ENDIF	
	ENDIF
	Sys14Count = m.Sys14Count+1
	Sys14Exp   = SYS(14, m.Sys14Count)
ENDDO
IF LEN(m.ReqStr)>1
	ReqStr=LEFT(m.ReqStr, LEN(m.ReqStr)-2)
ENDIF
RETURN m.ReqStr



********** GETALIAS - ALIASED FIELD TABLES - CURRENT TAG **********
FUNCTION GetAlias
PARAMETER InString
PRIVATE OutString, TempStr, X, C, WordIDX, CurWord

OutString=""
*** PARSE OUT OPERATORS/VALUES CONTAINING '.'s ***
TempStr=STRTRAN(STRTRAN(STRTRAN(STRTRAN(STRTRAN ;
	(UPPER(m.InString), " .AND. ", ' '), " .OR. ", ;
	' '), " .NOT. ", ' '), ".T.", ' '), ".F.", ' ')
*** REPLACE NON ALPHANUMERIC WITH ' ' - EXCEPT '_' AND '.' ***
FOR X=1 TO LEN(m.TempStr)
  C=SUBSTR(m.TempStr, m.X, 1)
  IF !(ISALPHA(m.C) OR ISDIGIT(m.C) OR ;
  		m.C="_" OR m.C='.')
    TempStr=STUFF(m.TempStr, m.X, 1, ' ')
  ENDIF
ENDFOR
*** FIND REMAINING VALID ALIASED TABLES ***
WordIDX = 1
CurWord = WORDNUM(m.TempStr, m.WordIdx)
DO WHILE !EMPTY(m.CurWord)
  IF '.' $ m.CurWord
    IF ISALPHA(m.CurWord) OR LEFT(m.CurWord, 1)='_'
      OutString=m.OutString+LEFT(m.CurWord, ;
      	AT('.', m.CurWord)-1)+"||"
    ENDIF
  ENDIF
	WordIDX = m.WordIDX+1
	CurWord = WORDNUM(m.TempStr, m.WordIdx)
ENDDO
RETURN m.OutString



********** BUILDTREE - BUILD RELATION TREE (CALLED RECURSIVELY) **********
PROCEDURE BuildTree
PRIVATE Cur_Area, Tgt_Index, RelTree

Cur_Area = SELECT(0)
RelTree  = "||"+ALIAS()

Tgt_Index = 1
DO WHILE !EMPTY(TARGET(m.Tgt_Index))
	SELECT (TARGET(m.Tgt_Index))
	RelTree=m.RelTree+BuildTree()												&& ==> Recursive Call
	SELECT (m.Cur_Area)
	Tgt_Index = m.Tgt_Index+1
ENDDO
RETURN m.RelTree
