****************************************************************************
*                                                                          *
* AUTHOR:       Richard Grossman (213) 547-2191                            *
*               Database Advisor April 92, Page 42                         *
*               Modified 02/94 by R. Scott Mackay                          *
*                                                                          *
* PROGRAM-ID:   Push_Set.PRG                                               *
*                                                                          *
* DESCRIPTION:  Allows you to set any On/OFF SET command, and save the     *
*               previous setting.  When you want to restore the previous   *
*               setting, just use Pop_Set.  Results are stored in a public *
*               variable 'gcSetList'.                                      *
*                                                                          *
* SYNTAX:       Push_Set(<expC1>[, <expC2>])                               *
* RETURNS:      Logical (was push valid/successful?)                       *
* PARAMETERS:   expC1 - SET Object  (i.e. 'BELL', 'CLOCK')                 *
*               expC2 - New Setting (i.e. 'ON', 'OFF')                     *
*                                                                          *
* EXAMPLE:      jlRetVal=Push_Set("CLOCK", "OFF")                          *
*                                                                          *
****************************************************************************
PARAMETERS lcObject, lcNewAct
PRIVATE lcOldAct

IF TYPE("m.lcObject")#'C' OR EMPTY(m.lcObject) ;
		OR EMPTY(SET(m.lcObject)) ;
		OR (TYPE("m.lcNewAct")#'C' ;
			AND TYPE("m.lcNewAct")#'L')
	RETURN .F.																					&& Invalid parameters
ENDIF

IF TYPE("gcSetList")='U'
	PUBLIC gcSetList
	gcSetList=""
ENDIF

lcObject = UPPER(ALLTRIM(m.lcObject))
lcNewAct = IIF(TYPE('m.lcNewAct')#'C', "", ;
	UPPER(ALLTRIM(m.lcNewAct)))

IF !EMPTY(m.lcNewAct) ;
		AND !INLIST(m.lcNewAct, "ON", "OFF")
	RETURN .F.																					&& Invalid values
ELSE
	lcOldAct  = IIF(SET(m.lcObject)=="ON", "ON ", "OFF")
	gcSetList = m.gcSetList+m.lcObject+'='+m.lcOldAct+','

	*** SET THE OBJECT TO ITS NEW STATE ***
	** Attempt to avoid macro expansion by testing for
	** some of the more common SET expressions.
	**
	DO CASE
	CASE m.lcObject=="ESCAPE"
		DO CASE
		CASE m.lcNewAct=="ON"
			SET ESCAPE ON
		CASE m.lcNewAct=="OFF"
			SET ESCAPE OFF
		ENDCASE
	CASE m.lcObject=="EXACT"
		DO CASE
		CASE m.lcNewAct=="ON"
			SET EXACT ON
		CASE m.lcNewAct=="OFF"
			SET EXACT OFF
		ENDCASE
	CASE m.lcObject=="NEAR"
		DO CASE
		CASE m.lcNewAct=="ON"
			SET NEAR ON
		CASE m.lcNewAct=="OFF"
			SET NEAR OFF
		ENDCASE
	CASE m.lcObject=="TALK"
		DO CASE
		CASE m.lcNewAct=="ON"
			SET TALK ON
		CASE m.lcNewAct=="OFF"
			SET TALK OFF
		ENDCASE
	OTHERWISE
		DO CASE
		CASE m.lcNewAct=="ON"
			SET &lcObject ON
		CASE m.lcNewAct=="OFF"
			SET &lcObject OFF
		ENDCASE
	ENDCASE
ENDIF
RETURN
